<?LassoScript

	// This file is the only file called at startup by Lasso.
	// This file must include all other files in startup.lassoapp
	// which need to be called.

	Auth_Admin;

	protect;
	
// Set the log level for all inlines within startup.lassoapp
	var: '__startup_loglevel__' = 'none';
	var: '__need_reload__' = false;
	
// Namespaces

	library:lassoapp_link:'namespace.lasso';

// Log Startup Message

	log_critical: 'Lasso Professional ' + (lasso_version: -lassoversion)->(substring: 1, (lasso_version: -lassoversion)->(find: ' ') - 1) + ' Started';
	log_critical: 'Site ' + (lasso_sitename) + ' (' + (lasso_siteid) + ') Started';

	var: '__startup_version__' = 'Unknown';
	protect;
		$__startup_version__ = (include: (lassoapp_link: 'version.txt'));
		log_warning: 'Startup Time Stamp: ' + (date: $__startup_version__);
		handle_error;
			error_code = 0;
			error_msg = '';
		/handle_error;
	/protect;
	$__startup_version__->freezevalue;
	$__startup_version__->freezetype;

// Load Lasso Tag Libraries

	Log_Warning: 'Lasso Tags: Defining Lasso 8 Tags';
	
	// Global variable tags [Global] [Global_Defined]
	library: (lassoapp_link: 'globals.lasso'); 

	// sort_arguments
	library:lassoapp_link:'sort.lasso';

	// Cache tags [Cache] ... [/Cache] [Cache_Object]
	if: (lasso_siteid != 0);
		library: (lassoapp_link: 'Cache.lasso'); 
	/if;

	// Security Tags
	library: (lassoapp_link: 'security.lasso'); 

	// Utility Tags [Named_Param] [Client_...] tags
	library: (lassoapp_link: 'utility.lasso'); 

	// Lasso [..._Serve] Tags
	library: (lassoapp_link: 'serveLibrary.lasso'); 

	// LassoApp_... Tags
	library: (lassoapp_link: 'lassoapp.lasso'); 

	// Comparators/Matchers
	library:lassoapp_link:'comparators.lasso';
	// generic iterator type
	library:lassoapp_link:'iterator.lasso';
	
	// SOAP and XML
	library:lassoapp_link:'namespaces.lasso';
	library:lassoapp_link:'xsd.lasso';
	if: !global('lp7_types.xsd')->isa('xml');
		global('lp7_types.xsd') = xml:include:lassoapp_link:'lp7types.xsd';
	/if;
	if: global('XML_SCHEMA_URIS')->size == 0;
		global('XML_SCHEMA_URIS') = set('http://www.w3.org/1999/XMLSchema', 'http://www.w3.org/2001/XMLSchema');
	/if;
	if: global('_xml_schema_cache')->size == 0;
		// this is a lengthy process (on Macs for some reason), so push it into its own thread
		global('_xml_schema_cache')->\deserialize->asAsync(-params=@array(@include:lassoapp_link:'XMLSchemaMap.xml'), -owner=@global('_xml_schema_cache'));	
	/if;
	library:lassoapp_link:'wsdl.lasso';
	library:lassoapp_link:'soap.lasso';
	
// Load Lasso Compatibility Tag Libraries

	Log_Warning: 'Lasso Tags: Defining Lasso Compatibility Tags';

	// Lasso 3 compatibility types [Repetition], [Client_Address] [Client_IP]
	library: (lassoapp_link: 'compatibility.lasso'); 

	// Lasso 3 compatibility tags [Math_ConvertEuro], etc.
	library: (lassoapp_link: 'MiscLibrary.lasso'); 
			
	// Locale tags
	library: (lassoapp_link: 'locale.lasso'); 
			
	// Locale tags
	library: (lassoapp_link: 'DatasourceIs.lasso'); 

	// Image tags (Stub if X11 is missing)
	if: (lasso_siteid != 0);
		library: (lassoapp_link: 'image_tags.lasso'); 
	/if;
	
// Check/Repair Databases

	library: (lassoapp_link: 'database_check.lasso'); 
	
// Create Utility Tables and Start Background Processes

	// Session tracker table in site database
	library: (lassoapp_link: 'sessiontrackerinit.lasso'); 

	// Create preferences table and define helper tags
	library: (lassoapp_link: 'LassoAdminStartup.lasso'); 

	// Software Update Notification check tag
	library: (lassoapp_link: 'sun_updatecheck.lasso'); 

	// Email Queue, SMTP, and POP tags
	library: (lassoapp_link: 'email/encode_crammd5.inc');
	library: (lassoapp_link: 'email/encode_quotedprintable.inc');
	library: (lassoapp_link: 'email/smtp.inc');
	library: (lassoapp_link: 'email/compose_send.inc');
	if: (lasso_siteid != 0);
		library: (lassoapp_link: 'email/pop_parse.inc');
		library: (lassoapp_link: 'email/smtp_queue.inc');
	/if;
	// Turn on Strict Syntax
	library: (lassoapp_link: 'StrictSyntax.lasso'); 
		
	// Schedule Queue and [Event_Schedule] tag
	if: (lasso_siteid != 0);
		library: (lassoapp_link: 'scheduleLibrary.lasso'); 
	/if;
	
	// XML_RPC type
	if: (lasso_siteid != 0);
		library: (lassoapp_link: 'xml_rpc.lasso');
	/if;
	
	// RPC Introspection
	if: (lasso_siteid != 0);
		library: (lassoapp_link: 'RPC_introspect.lasso'); 
	/if;
	
// Lasso Security

	// Load Lasso Security Modifications
	library: (lassoapp_link: 'LassoSecurity.lasso'); 

	// Refresh Lasso Security
	_security_flushcaches;

	// Load Tag Security
	library: (lassoapp_link: 'tag_security.lasso'); 

// Reload internal data source, and Dump Startup.LassoApp

	// Ensure that any altered database tables are loaded
	if: (global: '__need_reload__') == true;
		_datasource_reload: 1;
	/if;
	global_remove: '__need_reload__';
	
// Set Error Preferences (defaults used until this point)

	library: (lassoapp_link: 'errorSetup.lasso'); 

	admin_setpref: 'lasso_uptime', date;

	handle_error;
		log_critical: 'Startup Error: ' error_code ': ' error_msg;
	/handle_error;

	/protect;

	// Call lassoapp_dump asynchronously so it occurs after we finish executing.
	{ Sleep: 15000; LassoApp_Dump: 'Startup.LassoApp' }->AsAsync;

?>
