<?LassoScript

// Asynchronous SMTP Sweep

	{
		Protect;
		
			Log_Warning: 'SMTP: Initializing Email Queue';
		
			global: '__SMTP_Status__' = 'Active';
			global: '__SMTP_Sleep__' = (Date);
			global: '__SMTP_Queues__' = (Array);
			global: '__SMTP_Number__' = 1;
			global: '__SMTP_Defaults__' = False;
			
			// Check if the table exists
			Local: 'table_exists' = false;
			local: 'table_fields' = (map);
			Inline:
				-log=$__startup_loglevel__,
				-Database='site',
				-SQL='SELECT * from sqlite_master WHERE type="table" AND name="smtp_queue"';
				if: (found_count) > 0;
					#table_exists = true;
				/if;
			/inline;
		
			if: #table_exists == true;
				
				// Delete all records that are marked as non-restart or are past end date
				Inline:
					-log=$__startup_loglevel__,
					-Database='site',
					-SQL='DELETE FROM smtp_queue WHERE queue_status="delete"';
				/Inline;
				
				// Update all records with empty queue_number
				Inline:
					-log=$__startup_loglevel__,
					-Database='site',
					-SQL='UPDATE smtp_queue SET queue_number=0';
				/Inline;
										
			else;
				
				// Create table
				var: '__need_reload__' = true;
				Inline:
					-log=$__startup_loglevel__,
					-Database='site',
					-SQL='CREATE TABLE smtp_queue \
					(\
						id INTEGER, \
						
						id_site INTEGER, \
						
						mime_text TEXT, \
						mime_recipients TEXT, \
						mime_from TEXT, \
						
						smtp_host TEXT, \
						smtp_port INTEGER DEFAULT 25, \
						smtp_timeout INTEGER DEFAULT 15, \
						smtp_username TEXT, \
						smtp_password TEXT, \
						
						queue_date TEXT, \
						queue_status TEXT, \
						queue_error TEXT, \
						queue_number INTEGER DEFAULT 0, \
						queue_priority INTEGER DEFAULT 0, \
						queue_retries INTEGER DEFAULT 0, \
						
						create_date TEXT, \
						PRIMARY KEY (id) \
					)';
				/Inline;
			/if;
		
			handle_error;
				log_critical: 'SMTP: Unhandled Error (1): ' + error_msg + ' (' + error_code + ')';
			/handle_error;
		
		/protect;

	}->(Run);

	// The rest of the script is run asynchronously

	{
		Protect;
			
			Log_Warning: 'SMTP: Starting Email Queue';
			
			Local: 'prefs' = (map: 
					'lasso_smtp_pause' = 'N',
					'lasso_smtp_host' = '',
					'lasso_smtp_port' = 25,
					'lasso_smtp_username' = '',
					'lasso_smtp_password' = '',
					'lasso_smtp_timeout' = 15,
					'lasso_smtp_keep_error' = 'Y',
					'lasso_smtp_keep_success' = 'N',
					'lasso_smtp_keep_mime' = 'N',
					'lasso_smtp_sweep_delay' = 5,
					'lasso_smtp_resend_delay' = 600,
					'lasso_smtp_max_retries' = 5,
					'lasso_smtp_queue_count' = 5,
					'lasso_smtp_queue_size' = 100,
					'lasso_smtp_lassoservice_ip' = '',
				);					
			Local: 'prefs' = (admin_getpref: #prefs);
			Global: '__SMTP_Defaults__' = (#prefs->(find: 'lasso_smtp_host') != '');
			if: (#prefs->(find: 'lasso_smtp_lassoservice_ip') != '');
				global: '__smtp_ip__' = (#prefs->(find: 'lasso_smtp_lassoservice_ip'));
			else: (Global: '__lassoservice_ip__') != '';
				global: '__smtp_ip__' = (Global: '__lassoservice_ip__');
			else;
				global: '__smtp_ip__' = '127.0.0.1';
			/if;
	
			if: (#prefs->(find: 'lasso_smtp_pause') == 'Y');
				global: '__SMTP_Status__'='Pause';
			else;
				global: '__SMTP_Status__'='Active';
			/if;
				
			// 15 Second Delay at Startup
			global: '__SMTP_Status__' = 'Abort';
			Sleep: 15000; 
			global: '__SMTP_Status__' = 'Active';
			
			While: ((Global: '__SMTP_Status__') != 'Abort');
	
				// Delay Between Active Checks
				Sleep: ((math_range: (integer: #prefs->(find: 'lasso_smtp_sweep_delay')), 1, 3600) * 1000); 
				
				If: ((Global: '__SMTP_Status__') >> 'Refresh');
					Local: 'prefs' = (admin_getpref: #prefs);
					Global: '__SMTP_Defaults__' = (#prefs->(find: 'lasso_smtp_host') != '');
					if: (#prefs->(find: 'lasso_smtp_pause') == 'Y');
						global: '__SMTP_Status__'='Pause';
					else;
						global: '__SMTP_Status__'='Active';
					/if;

					if: (#prefs->(find: 'lasso_smtp_lassoservice_ip') != '');
						global: '__smtp_ip__' = (#prefs->(find: 'lasso_smtp_lassoservice_ip'));
					else: (Global: '__lassoservice_ip__') != '';
						global: '__smtp_ip__' = (Global: '__lassoservice_ip__');
					else;
						global: '__smtp_ip__' = '127.0.0.1';
					/if;
				/If;
				
				If: ((Global: '__SMTP_Status__') >> 'Pause') || ((Global: '__SMTP_Status__') >> 'Stop');
					Loop_Continue;
				else: ((Global: '__SMTP_Status__') >> 'Sleep') && (!(Global: '__SMTP_Sleep__')->(isa: 'date') || ((Global: '__SMTP_Sleep__') > Date));
					Loop_Continue;
				/if;
				
				Protect;
					Log_Detail: 'SMTP: Checking Email Queue.';
						
					// Queue Operations
					if: ((global: '__SMTP_Queues__')->size < (math_range: (integer: #prefs->(find: 'lasso_smtp_queue_count')), 1, 25));
						inline:
								-log=$__startup_loglevel__,
								-database='site',
								-sql='SELECT * FROM smtp_queue \
									WHERE \
										queue_status = "queued" AND \
										queue_number = 0 AND \
										queue_date < datetime(\'now\',\'localtime\') \
									ORDER BY queue_priority \
									LIMIT 1';
	
							if: (found_count > 0);
								local: 'number' = (integer: (global: '__SMTP_Number__')) + 1;
								global: '__SMTP_Number__' = #number;
							
								inline:
										-log=$__startup_loglevel__,
										-database='site',
										-sql='SELECT id FROM smtp_queue \
											WHERE \
												queue_status = "queued" AND \
												queue_date < datetime(\'now\',\'localtime\') AND \
												queue_number = 0 AND \
												smtp_host = "' + (field: 'smtp_host') + '" AND \
												smtp_port = "' + (field: 'smtp_port') + '" AND \
												smtp_username = "' + (field: 'smtp_username') + '" AND \
												smtp_password = "' + (field: 'smtp_password') + '" \
											ORDER BY queue_priority, queue_date \
											LIMIT ' + (math_range: (integer: #prefs->(find: 'lasso_smtp_queue_size')), 1, 200);
										local: 'ids' = (array);
										records;
											#ids->(insert: (field: 'id'));
										/records;
										inline:
												-log=$__startup_loglevel__,
												-database='site',
												-sql='UPDATE smtp_queue \
													SET queue_number = ' + #number + ' \
													WHERE id IN (' + #ids->(join: ',') + ')';
										/inline;
								/inline;
							
								// Queue Object
								{
									Protect;
									
										local: 'number' = params->first;
										local: 'prefs' = params->second;
										local: 'makeactive' = false;
										
										(global: '__SMTP_Queues__')->(insert: #number = (date));
										
										inline:
												-log=$__startup_loglevel__,
												-database='site',
												-sql='SELECT * FROM smtp_queue \
													WHERE \
														queue_status = "queued" AND \
														queue_date < datetime(\'now\',\'localtime\') AND \
														queue_number = "' + #number + '" \
													ORDER BY mime_recipients',
												-maxrecords='all';
											
											if: (found_count > 0);
											
												if: (found_count == 1);
													Log_Detail: 'SMTP: Sending 1 Email Message For Queue #' + #number + '.';
												else;
													Log_Detail: 'SMTP: Sending ' + (found_count) + ' Email Messages For Queue #' + #number + '.';
												/if;
												
												local: 'continue' = true;

												// Open SMTP connection
												if: ((field: 'smtp_host') != '');
													// Use all field values if host is set
													local: 'smtp_host' = (field: 'smtp_host');
													local: 'smtp_port' = (field: 'smtp_port');
													local: 'smtp_username' = (field: 'smtp_username');
													local: 'smtp_password' = (field: 'smtp_password');
													local: 'smtp_timeout' = (field: 'smtp_timeout');
												else;
													// Otherwise use all preference values
													local: 'smtp_host' = #prefs->(find: 'lasso_smtp_host');
													local: 'smtp_port' = #prefs->(find: 'lasso_smtp_port');
													local: 'smtp_username' = #prefs->(find: 'lasso_smtp_username');
													local: 'smtp_password' = #prefs->(find: 'lasso_smtp_password');
													local: 'smtp_timeout' = #prefs->(find: 'lasso_smtp_timeout');
												/if;
												
												// Reality Check
												if: ((integer: #smtp_port) <= 0);
													local: 'smtp_port' = 25;
												/if;
												if: ((integer: #smtp_timeout) <= 0);
													local: 'smtp_timeout' = 15;
												/if;
												
												if: (#smtp_host == '');
													local: 'continue' = false;
													local: 'smtp_error' = 'SMTP: Lasso is not configured with a default host.';
													local: 'makeactive' = true;
													error_code = -1;
													error_msg = #smtp_error;
													log_critical: 'SMTP: ' + #smtp_error + ' (' + error_code + ')';
												/if;
												
												if: (#continue == true);
													protect;
														local: 'smtp' = (email_smtp);
														handle_error;
															local: 'continue' = false;
															local: 'smtp_error' = 'SMTP: Failed to initialize SMTP connector.';
															local: 'makeactive' = true;
															log_critical: 'SMTP: ' + error_msg + ' (' + error_code + ')';
														/handle_error;
													/protect;
												/if;
												if: (#continue == true);
													protect;
														if: (#smtp_username != '') && (#smtp_password != '');
															#smtp->(open: -host=#smtp_host, -port=(integer: #smtp_port), -username=#smtp_username, -password=#smtp_password);
														else;
															#smtp->(open: -host=#smtp_host, -port=(integer: #smtp_port));
														/if;
														handle_error;
															local: 'continue' = false;
															local: 'smtp_error' = 'SMTP: Failed to connect to host "' + #smtp_host + '". ' + error_msg + ' (' + error_code + ')';
															local: 'makeactive' = true;
															log_critical: #smtp_error;
														/handle_error;
													/protect;
												/if;
												records;
													// default values
													local: 'queue_status' = 'queued';
													local: 'queue_retries' = (integer: (field: 'queue_retries'));
													local: 'queue_error' = (field: 'queue_error');
													local: 'queue_date' = date->(format: '%Q %T');
													
													// Send each message
													if: ((field: 'mime_from') == '');
														local: 'queue_status' = 'error';
														local: 'queue_error' = 'Message does not have a valid from address.\r\n' + #queue_error;
													else: ((field: 'mime_recipients') == '');
														local: 'queue_status' = 'error';
														local: 'queue_error' = 'Message does not have any valid recipients.\r\n' + #queue_error;
													else: ((field: 'mime_text') == '');
														local: 'queue_status' = 'error';
														local: 'queue_error' = 'Message does not have a valid body or parts.\r\n' + #queue_error;
													else: (#continue == true);
														protect;
															local: 'message' = (field: 'mime_text');
															#message += '\r\n';
															#smtp->(send: -from=(field: 'mime_from'), -recipients=(field: 'mime_recipients')->(split: ','), -message=#message);
															local: 'queue_status' = 'sent';
															handle_error;
																if: ((string: error_msg) >> 'SMTP') && 
																	( (((integer: error_code) >= 100) && ((integer: error_code) < 200)) ||
																	(((integer: error_code) >= 300) && ((integer: error_code) < 1000)) );
																	// Non-200 Errors are Fatal
																	local: 'queue_status' = 'error';
																	local: 'queue_retries'= #queue_retries + 1;
																	local: 'queue_error' = error_msg + ' (' + error_code + ')' + '\r\n' + #queue_error;
																	local: 'queue_date' = (date)->(format: '%Q %T');
																	log_critical: 'SMTP: ' + error_msg + ' (' + error_code + ')';
																else;
																	local: 'queue_status' = 'queued';
																	local: 'queue_retries'= #queue_retries + 1;
																	local: 'queue_error' = error_msg + ' (' + error_code + ')' + '\r\n' + #queue_error;
																	local: 'queue_date' = (date_add: #queue_date, -second=(math_range: (integer: #prefs->(find: 'lasso_smtp_resend_delay')), 60, 3600))->(format: '%Q %T');
																	log_critical: 'SMTP: ' + error_msg + ' (' + error_code + ')';
																	local: 'makeactive' = true;
																/if;
															/handle_error;
														/protect;
													else: (local_defined: 'smtp_error');
														if: #smtp_error >> 'at:';
															local: 'smtp_error' = #smtp_error->(substring: 0, #smtp_error->(find: 'at:') - 1);
															#smtp_error->trim;
														/if;
														local: 'queue_status' = 'queued';
														local: 'queue_retries'= #queue_retries + 1;
														local: 'queue_error' = #smtp_error + '\r\n' + #queue_error;
														local: 'queue_date' = (date_add: #queue_date, -second=(math_range: (integer: #prefs->(find: 'lasso_smtp_resend_delay')), 60, 3600))->(format: '%Q %T');
														local: 'makeactive' = true;
													/if;
													
													// Decide if we want to delete the message or not
													if: (#queue_retries > (math_range: (integer: #prefs->(find: 'lasso_smtp_max_retries')), 1, 100));
														local: 'queue_status' = 'error';
													/if;
													
													local: 'inline' = (array:
															-update = '',
															-database='site',
															-table='smtp_queue',
															-keyfield='id',
															-keyvalue=(field: 'id'),
															'queue_status' = #queue_status,
															'queue_date' = #queue_date,
															'queue_retries' = #queue_retries,
															'queue_error' = #queue_error,
															'queue_number' = 0,
														);
																										
													if: (#queue_status == 'error') && (#prefs->(find: 'lasso_smtp_keep_error') >> 'N') ||
														(#queue_status == 'sent') && (#prefs->(find: 'lasso_smtp_keep_success') >> 'N');
														
														local: 'inline' = (array:
																-delete = '',
																-database='site',
																-table='smtp_queue',
																-keyfield='id',
																-keyvalue=(field: 'id'),
															);
													
													else: (#queue_status == 'sent') && (#prefs->(find: 'lasso_smtp_keep_mime') >> 'N');
													
														#inline->(insert: 'mime_text' = (string_replaceregexp: (field: 'mime_text'), -find='\r\n\r\n[\\S\\s]*', -replace='\r\n\r\n'));
														
													/if;
														
													inline:
														-log=$__startup_loglevel__,
														#inline;
													/inline;
														
												/records;
												
												protect;
													#smtp->close;
												/protect;
											/if;
										
										/inline;
									
										handle: true;
											protect;
												#smtp->'net'->close;
											/protect;
										/handle;
										
										(global: '__SMTP_Queues__')->(removeall: #number);
										
										Log_Detail: 'SMTP: Done Sending Messages For Queue #' + #number + '.';
										
										if: (#makeactive == true) && ((Global: '__SMTP_Status__') !>> 'Pause') && ((Global: '__SMTP_Status__') !>> 'Stop');
											global: '__SMTP_Status__' = 'Active';
										/if;
										
										handle_error;
											log_critical: 'SMTP: Unhandled Error (2): ' + error_msg + ' (' + error_code + ')';
										/handle_error;
										
									/Protect;
									
								}->(AsAsync: -Params=(array: #number, #prefs));
							
							/if;
						/inline;
					/if;
	
					if: ((Global: '__SMTP_Status__') != 'Pause') && ((Global: '__SMTP_Status__') != 'Stop');
						// Check for unsent messages
						inline:
								-log=$__startup_loglevel__,
								-database='site',
								-sql='SELECT * FROM smtp_queue \
									WHERE \
										queue_status = "queued" AND \
										queue_number = 0 AND \
										queue_date <= datetime(\'now\',\'localtime\') \
									LIMIT 1';
							if: (found_count) > 0;
								global: '__SMTP_Status__' = 'Active';
							else;
								inline:
										-log=$__startup_loglevel__,
										-database='site',
										-sql='SELECT * FROM smtp_queue \
											WHERE \
												queue_status = "queued" AND \
												queue_number = 0 AND \
												queue_date > datetime(\'now\',\'localtime\') \
											ORDER BY queue_date \
											LIMIT 1';
									global: '__SMTP_Status__' = 'Sleep';
									global: '__SMTP_Sleep__' = (date_add: date, -hour=1);
									if: ((found_count) > 0) && ((field: 'queue_date') != '');
										if: (date: (field: 'queue_date')) < (date_add: date, -hour=1);
											global: '__SMTP_Sleep__' = (date: (field: 'queue_date'));
										/if;
									else;
										Log_Detail: 'SMTP: Sleeping Until ' + (global: '__SMTP_Sleep__');
									/if;
								/inline;
							/if;
						/inline;
					/if;
					
					handle_error;
						log_critical: 'SMTP: Unhandled Error (3): ' + error_msg + ' (' + error_code + ')';
					/handle_error;
	
				/Protect;
							
			/While;
			
			Log_Warning: 'SMTP: Stopping Email Queue';	
	
			handle_error;
				log_critical: 'SMTP: Unhandled Error (4): ' + error_msg + ' (' + error_code + ')';
			/handle_error;

		/Protect;

	}->(AsAsync); 
?>
