<?LassoScript

	/*
		Library of routines to perform interesting TCP functions like [Stock_Quote]

		History:
		06/23/2004	AJR - Rewrote the tag to use the NET tags rather than the TCP_... tags.
		06/08/2004	FMS - Moved to on-demand library with namespaces
		10/17/2001	GLH - moved return value to prevent leakage of 9 handles on Windows
		01/17/2001	GLH - use new substring search syntax
		          	GLH - use new parameter-passing syntax
		          	GLH - change Tag to Define_Tag
		11/22/2000	GLH - Initial work
	*/


	/*
		Problems:

		Inputs:
			-Ticker:	string, such as "AAPL" or "SMSI" indicates which stock to get info for

		Outputs:
			Text from Yahoo stock-quote engine, dollar value of stock requested
	*/


	Define_Tag('Quote',
		-Required='Ticker',
		-NameSpace='_global_Stock',
		-priority='replace');

		local
		(
			'timeout' = 15,	// 15-second TCPIP connection timeout
			'n' = net,
			'result'
		);

		if(#n->connect('quote.yahoo.com', 80, #timeout) == net_connectok);
			#result = #n->wait(#timeout, net_waitwrite);
			if(#result->isa('integer') && (#result == net_waittimeout));
				fail(-1, '[Stock_Quote] Timed out waiting for write.');
			/if;

			#result = #n->write('GET /d/quotes.csv?s=' #ticker '&f=l1 HTTP/1.0\r\n\r\n');

			#result = #n->wait(#timeout, net_waitread);
			if(#result->isa('integer') && (#result == net_waittimeout));
				fail(-1, '[Stock_Quote] Timed out waiting for read.');
			/if;

			#result = #n->read(32768)->exportstring('iso-8859-1');
			#result = #result->substring(#result->find('\r\n\r\n') + 4, #result->length);
			#result->trim;
			#result = decimal(#result);

			protect;
				#n->close;
			/protect;

			return(@#result);

		else;
			fail(-1, '[Stock_Quote] Could not connect to server.');
		/if;

		Handle(True);
			protect;
				#n->close;
			/protect;
		/Handle;

		Return(#result);

	/Define_Tag;

?>